@@ -1,4 +1,6 @@
+import collections
 import json
+import multiprocessing
 import os
 import pathlib
 from datetime import datetime
@@ -21,6 +23,60 @@ from examtool.cli.utils import (
 )
 
 
+def do_compile(args):
+    (
+        email,
+        deadline,
+        no_watermark,
+        exam,
+        out,
+        subtitle,
+        do_twice,
+        latex_prefix,
+        latex_suffix,
+        exam_type,
+        semester,
+        exam_str,
+        password,
+    ) = args
+
+    if not deadline:
+        return
+    exam_data = json.loads(exam_str)
+    scramble(email, exam_data)
+    if no_watermark:
+        exam_data.pop("watermark")
+    deadline_utc = datetime.utcfromtimestamp(int(deadline))
+    deadline_pst = pytz.utc.localize(deadline_utc).astimezone(
+        pytz.timezone("America/Los_Angeles")
+    )
+    deadline_string = deadline_pst.strftime("%I:%M%p")
+
+    with render_latex(
+        exam_data,
+        {
+            "emailaddress": sanitize_email(email),
+            "deadline": deadline_string,
+            "coursecode": prettify(exam.split("-")[0]),
+            "description": subtitle,
+            "examtype": exam_type,
+            "semester": semester,
+        },
+        do_twice=do_twice,
+        prefix=latex_prefix,
+        suffix=latex_suffix,
+        suppress_output=True,
+    ) as pdf:
+        pdf = Pdf.open(BytesIO(pdf))
+        pdf.save(
+            os.path.join(
+                out, "exam_" + email.replace("@", "_").replace(".", "_") + ".pdf"
+            ),
+            encryption=Encryption(owner=password, user=password),
+        )
+        pdf.close()
+
+
 @click.command()
 @exam_name_option
 @hidden_output_folder_option
@@ -62,12 +118,6 @@ from examtool.cli.utils import (
     default=None,
     help="Generates exam regardless of if student is in roster with the set deadline.",
 )
-@click.option(
-    "--show-pbar",
-    default=False,
-    is_flag=True,
-    help="Suppresses LaTeX compilation output and displays a progress bar instead.",
-)
 def compile_all(
     exam,
     out,
@@ -79,7 +129,6 @@ def compile_all(
     exam_type,
     semester,
     deadline,
-    show_pbar,
 ):
     """
     Compile individualized PDFs for the specified exam.
@@ -110,46 +159,50 @@ def compile_all(
             else:
                 raise ValueError("Email does not exist in the roster!")
 
-    if show_pbar:
-        roster = tqdm(roster)
-    for email, deadline, no_watermark in roster:
-        if show_pbar:
-            roster.set_description(f"compiling {email}")
-        if not deadline:
-            continue
-        exam_data = json.loads(exam_str)
-        scramble(email, exam_data)
-        if no_watermark:
-            exam_data.pop("watermark")
-        deadline_utc = datetime.utcfromtimestamp(int(deadline))
-        deadline_pst = pytz.utc.localize(deadline_utc).astimezone(
-            pytz.timezone("America/Los_Angeles")
+    if exam_data.get("watermark") and not do_twice:
+        print(
+            "!!! WARNING: your exam seems to contain a watermark, but you're not using the --do-twice option;"
+        )
+        print(
+            "!!! due to some underlying latex issues, this may cause your watermarks to not render properly,"
+        )
+        print(
+            "!!! so we suggest using the --do-twice flag to ensure that your watermarks are rendered."
         )
-        deadline_string = deadline_pst.strftime("%I:%M%p")
 
-        with render_latex(
-            exam_data,
-            {
-                "emailaddress": sanitize_email(email),
-                "deadline": deadline_string,
-                "coursecode": prettify(exam.split("-")[0]),
-                "description": subtitle,
-                "examtype": exam_type,
-                "semester": semester,
-            },
-            do_twice=do_twice,
-            prefix=latex_prefix,
-            suffix=latex_suffix,
-            suppress_output=show_pbar,
-        ) as pdf:
-            pdf = Pdf.open(BytesIO(pdf))
-            pdf.save(
-                os.path.join(
-                    out, "exam_" + email.replace("@", "_").replace(".", "_") + ".pdf"
+    reused_args = (
+        exam,
+        out,
+        subtitle,
+        do_twice,
+        latex_prefix,
+        latex_suffix,
+        exam_type,
+        semester,
+        exam_str,
+        password,
+    )
+    with multiprocessing.Pool() as pool:
+        # The deque with maxlen=0 is just to consume the tqdm iterator.
+        collections.deque(
+            tqdm(
+                pool.imap(
+                    do_compile,
+                    (
+                        (
+                            email,
+                            deadline,
+                            no_watermark,
+                            *reused_args,
+                        )
+                        for email, deadline, no_watermark in roster
+                    ),
+                    chunksize=1,
                 ),
-                encryption=Encryption(owner=password, user=password),
-            )
-            pdf.close()
+                total=len(roster),
+            ),
+            maxlen=0,
+        )
 
 
 if __name__ == "__main__":
